/*
 * 
 * The Seasar Software License, Version 1.1
 *
 * Copyright (c) 2003-2004 The Seasar Project. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or 
 * without modification, are permitted provided that the following 
 * conditions are met:
 *
 * 1. Redistributions of source code must retain the above 
 *    copyright notice, this list of conditions and the following 
 *    disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above 
 *    copyright notice, this list of conditions and the following 
 *    disclaimer in the documentation and/or other materials provided 
 *    with the distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgement:  
 *    "This product includes software developed by the 
 *    Seasar Project (http://www.seasar.org/)."
 *    Alternately, this acknowledgement may appear in the software
 *    itself, if and wherever such third-party acknowledgements 
 *    normally appear.
 *
 * 4. Neither the name "The Seasar Project" nor the names of its
 *    contributors may be used to endour or promote products derived 
 *    from this software without specific prior written permission of 
 *    the Seasar Project.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR 
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE SEASAR PROJECT 
 * OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 * INCIDENTAL,SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING 
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.seasar.kijimuna.ui.dicon.provider.property;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.seasar.kijimuna.core.dicon.info.IComponentKey;
import org.seasar.kijimuna.core.dicon.info.IComponentNotFound;
import org.seasar.kijimuna.core.dicon.info.ITooManyRegisted;
import org.seasar.kijimuna.core.rtti.IRtti;
import org.seasar.kijimuna.core.rtti.IRttiInvokableDesctiptor;
import org.seasar.kijimuna.core.util.ModelUtils;
import org.seasar.kijimuna.ui.KijimunaUI;

/**
 * @author Masataka Kurihara (Gluegent, Inc.)
 */
public class AutoInjectedArgProperty extends NullProperty {

    private static IPropertyDescriptor[] descriptors;
    
    static {
        // info category
        String category = KijimunaUI.getResourceString(
                "dicon.provider.property.ContentProperty.4");
        descriptors = new IPropertyDescriptor[5];
        // method name
        String id = "dicon.provider.property.AutoInjectedArgProperty.1";
        PropertyDescriptor d = 
            new PropertyDescriptor(id, KijimunaUI.getResourceString(id));
        d.setCategory(category);
        descriptors[0] = d;
        // return type
        id = "dicon.provider.property.AutoInjectedArgProperty.2";
        d = new PropertyDescriptor(id, KijimunaUI.getResourceString(id));
        d.setCategory(category);
        descriptors[1] = d;
        // arg index
        id = "dicon.provider.property.AutoInjectedArgProperty.3";
        d = new PropertyDescriptor(id, KijimunaUI.getResourceString(id));
        d.setCategory(category);
        descriptors[2] = d;
        // arg type
        id = "dicon.provider.property.AutoInjectedArgProperty.4";
        d = new PropertyDescriptor(id, KijimunaUI.getResourceString(id));
        d.setCategory(category);
        descriptors[3] = d;
        // injected component
        id = "dicon.provider.property.AutoInjectedArgProperty.5";
        d = new PropertyDescriptor(id, KijimunaUI.getResourceString(id));
        d.setCategory(category);
        descriptors[4] = d;
    }
	
    private IRttiInvokableDesctiptor method;
    private int index;
	
	public AutoInjectedArgProperty(IRttiInvokableDesctiptor method, int index) {
		this.method = method;
		this.index = index;
	}
	
	public IPropertyDescriptor[] getPropertyDescriptors() {
		return descriptors;
	}

	public Object getPropertyValue(Object id) {
		if("dicon.provider.property.AutoInjectedArgProperty.1".equals(id)) {
		    return ModelUtils.getMethodDisplay(method, true);
		} else if("dicon.provider.property.AutoInjectedArgProperty.2".equals(id)) {
		    return method.getReturnType().getQualifiedName();
		} else if("dicon.provider.property.AutoInjectedArgProperty.3".equals(id)) {
		    return new Integer(index);
		} else if("dicon.provider.property.AutoInjectedArgProperty.4".equals(id)) {
		    return method.getArgs()[index].getQualifiedName();
		} else if("dicon.provider.property.AutoInjectedArgProperty.5".equals(id)) {
		    IRtti value = method.getValues()[index];
		    if(value instanceof IComponentNotFound) {
		        return KijimunaUI.getResourceString(
		                "dicon.provider.property.AutoInjectedArgProperty.6",
		                new Object[] { value.getAdapter(IComponentKey.class) });
		    } else if(value instanceof ITooManyRegisted) {
		        return ((ITooManyRegisted)value).getErrorMessage();
		    } else {
	            return ModelUtils.getInjectedElementName(value);
		    }
		} 
		return null;
	}

}
